@@ -6,10 +6,10 @@ require_once 'Horde/History.php';
 /**
  * The Group:: class provides the Horde groups system.
  *
- * $Horde: framework/Group/Group.php,v 1.67.2.15 2006/07/17 14:40:19 jan Exp $
+ * $Horde: framework/Group/Group.php,v 1.67.2.16.2.1 2008/03/12 17:35:31 chuck Exp $
  *
- * Copyright 1999-2006 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -397,7 +397,7 @@ class Group {
             $groups = $this->_datatree->getByAttributes($criteria);
 
             if (is_a($groups, 'PEAR_Error')) {
-                Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERROR);
+                Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERR);
                 return false;
             }
 
