@@ -7,20 +7,21 @@ require_once 'Horde/String.php';
  * Highlights quoted messages with different colors for the different quoting
  * levels.
  *
- * CSS class names called "quoted1" .. "quoted$level" must be present.  The
- * text to be passed in must have already been passed through
+ * CSS class names called "quoted1" ... "quoted{$cssLevels}" must be present.
+ * The text to be passed in must have already been passed through
  * htmlspecialchars().
  *
  * Parameters:
  * <pre>
+ * 'citeblock'  -- Display cite blocks? Defaults to true.
+ * 'cssLevels'  -- Number of defined CSS class names. Defaults to 5.
  * 'hideBlocks' -- Hide quoted text blocks by default? Defaults to false.
- * 'citeblock'  -- Display cite blocks? Defaults to true,
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.3 2005/06/23 19:57:41 selsky Exp $
+ * $Horde: framework/Text_Filter/Filter/highlightquotes.php,v 1.6.8.17.2.1 2008/05/18 11:32:21 jan Exp $
  *
- * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2007 Michael Slusarz <slusarz@curecanti.org>
+ * Copyright 2004-2007 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -37,7 +38,11 @@ class Text_Filter_highlightquotes extends Text_Filter {
      *
      * @var array
      */
-    var $_params = array('hideBlocks' => false, 'citeblock' => true);
+    var $_params = array(
+        'citeblock' => true,
+        'cssLevels' => 5,
+        'hideBlocks' => false
+    );
 
     /**
      * Executes any code necessaray before applying the filter patterns.
@@ -79,62 +84,97 @@ class Text_Filter_highlightquotes extends Text_Filter {
      */
     function postProcess($text)
     {
-        /* CSS defines 5 different colors for quoting. */
-        $level = 5;
-
-        /* Add show/hide toggles for whole quote blocks. */
-        $text = preg_replace_callback("/(\n)(( *(&gt;\s?)+(?! ?&gt;).*?)(\n|$)(?! *(&gt; ?)+))/s",
-                                      array($this, '_addQuoteToggles'), $text);
-
-        /* Use cite blocks to display the different quoting levels. */
+        /* Is there a blank line(s) before the current line? */
         $blankline = false;
+
+        /* Use cite blocks to display the different quoting levels? */
         $cb = $this->_params['citeblock'];
+
+        /* Cite level before parsing the current line. */
         $qlevel = 0;
+
+        /* Generated output. */
         $text_out = '';
 
+        /* As of PHP 5.2, backtrack limits have been set to an unreasonably
+         * low number. The body check will often times trigger backtrack
+         * errors so up the backtrack limit if we are doing this match. */
+        if (ini_get('pcre.backtrack_limit')) {
+            ini_set('pcre.backtrack_limit', 5000000);
+        }
+
+        /* Add show/hide toggles for whole quote blocks. */
+        $text = preg_replace_callback(
+            '/(\n)(( *(&gt;\s?)+(?! ?&gt;).*?)(\n|$)(?! *(&gt; ?)+))/s',
+            array($this, '_addQuoteToggles'), $text);
+
+        ini_restore('pcre.backtrack_limit');
+
+        /* Parse text line by line. */
         foreach (explode("\n", $text) as $line) {
-            $clevel = 0;
+            /* Cite level of current line. */
+            $clevel = $closelevel = 0;
             $matches = array();
-            if (preg_match("/^\s*((&gt;\s?)+)/m", $line, $matches)) {
-                $clevel = count(preg_split('/&gt;\s?/', $matches[1])) - 1;
+            /* Do we have a citation line? */
+            if (preg_match('/^\s*((&gt;\s?)+)/m', $line, $matches)) {
+                /* Count number of > characters => cite level */
+                $clevel = $closelevel = count(preg_split('/&gt;\s?/', $matches[1])) - 1;
             }
 
             if ($cb && isset($matches[1])) {
+                /* Strip all > characters. */
                 $line = substr($line, String::length($matches[1]));
             }
 
+            /* Is this cite level lower than the current level? */
             if ($clevel < $qlevel) {
+                /* Strip more than one blank line in front of the cite
+                 * blocks. */
+                if ($blankline) {
+                    while (($br_str = substr($text_out, -7)) == "<br />\n") {
+                        $text_out = substr($text_out, 0, -7);
+                    };
+                }
+                /* Add quote block end tags for each cite level. */
                 for ($i = $clevel; $i < $qlevel; $i++) {
                     $text_out .= ($cb) ? '</div>' : '</font>';
+                    --$closelevel;
                 }
-                if ($cb && preg_match("/\<br \/>$/", $line)) {
+                /* Strip trailing line break if using cite blocks, we already
+                 * have a div tag. */
+                if ($cb && preg_match('/\<br \/>$/', $line)) {
                     $line = substr($line, 0, -6);
                 }
+            /* Is this cite level higher than the current level? */
             } elseif ($clevel > $qlevel) {
+                /* Strip more than one blank line in front of the cite
+                 * blocks. */
                 if ($blankline) {
-                    $text_out = substr(rtrim($text_out), 0, -6);
+                    while (($br_str = substr($text_out, -7)) == "<br />\n") {
+                        $text_out = substr($text_out, 0, -7);
+                    };
                 }
                 for ($i = $qlevel; $i < $clevel; $i++) {
-                    if ($cb) {
-                        $text_out .= '<div class="citation quoted' . (($i % $level) + 1) . '">';
-                    } else {
-                        $text_out .= '<font class="quoted' . (($i % $level) + 1) . '">';
-                    }
+                    /* Add quote block start tags for each cite level. */
+                    $text_out .= ($cb) ? '<div class="citation ' : '<font class="';
+                    $text_out .= 'quoted' . (($i % $this->_params['cssLevels']) + 1) . '">';
                 }
             }
 
-            if ($line == '<br />') {
-                $blankline = true;
-            } else {
-                $blankline = false;
-            }
+            /* Count blank lines. */
+            $blankline = ($line == '<br />');
 
             $text_out .= $line . "\n";
             $qlevel = $clevel;
         }
 
+        /* Make sure all div/font tags are closed. */
+        if ($closelevel > 0) {
+            $text_out = preg_replace('/<\/div>$/', str_repeat(($cb) ? '</div>' : '</font>', $closelevel) . '</div>', $text_out);
+        }
+
         /* Remove the leading newline we added above, if it's still there. */
-        if ($text_out{0} == "\n") {
+        if ($text_out[0] == "\n") {
             $text_out = substr($text_out, 1);
         }
 
@@ -168,7 +208,7 @@ class Text_Filter_highlightquotes extends Text_Filter {
             $text = '<br />';
         };
         $text .= '<div id="qt_' . $i . '">' .
-            Horde::link('#', '', 'widget', '', 'toggleQuoteBlock(\'' . $i . '\', \'' . $lines . '\'); return false;', '', '', array('style' => 'font-size:70%')) .
+            Horde::link('#', '', 'widget togglequote', '', 'toggleQuoteBlock(\'' . $i . '\', \'' . $lines . '\'); return false;', '', '', array('style' => 'font-size:70%')) .
             ($this->_params['hideBlocks'] ? sprintf(_("[Show Quoted Text - %s lines]"), $lines) : _("[Hide Quoted Text]")) .
             '</a></div><div id="qb_' . $i . '"' . ($this->_params['hideBlocks'] ? ' style="display:none;"' : '') . '>' . $matches[0] . '</div>';
 
